Imports System.Data.OleDb

Public Class Form1
    ' The database connection.
    Private m_Connection As OleDbConnection

    ' The fields in the database's Books table.
    Private m_Fields() As String

    ' True until we finish loading.
    Dim m_Loaded As Boolean = False

    ' Connect to the database.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Set some properties. 
        ' (I do this here to make it easier to find them.)
        dgvBooks.AutoGenerateColumns = True
        dgvBooks.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells

        ' Prepare the OleDbConnection object for use.
        PrepareConnection()

        ' Get the data.
        SelectData("SELECT * FROM Books ORDER BY Title")

        ' Restore the saved column visibilities.
        LoadSettings()

        m_Loaded = True
    End Sub

    ' Prepare the OleDbConnection object for use.
    Private Sub PrepareConnection()
        ' Compose the database path.
        Dim db_path As String = Application.StartupPath
        If db_path.EndsWith("\bin\Debug") OrElse db_path.EndsWith("\bin\Release") Then
            db_path = db_path.Substring(0, db_path.LastIndexOf("\bin"))
        End If
        If Not db_path.EndsWith("\") Then db_path &= "\"
        db_path &= "Books.mdb"

        ' Prepare the connection.
        m_Connection = New OleDbConnection( _
            "Provider=Microsoft.Jet.OLEDB.4.0;" & _
            "Data Source=""" & db_path & """" _
        )
    End Sub

    ' Display data.
    Private Sub SelectData(ByVal query As String)
        Debug.WriteLine(query)

        ' Open the connection.
        Try
            m_Connection.Open()
        Catch ex As Exception
            MessageBox.Show( _
                "Error opening database using connect string:" & _
                    vbCrLf & vbCrLf & _
                    vbTab & m_Connection.ConnectionString & vbCrLf & vbCrLf & _
                    ex.Message & vbCrLf & vbCrLf & "Closing the application.", _
                "Connection Error", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Stop)
            End
        End Try

        ' Select the data.
        Dim data_table As New DataTable("Books")
        Dim da As New OleDbDataAdapter(query, m_Connection)
        Try
            ' Get the data.
            da.Fill(data_table)

            ' Save the query.
            SaveQuery(query)
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Query Error", _
                MessageBoxButtons.OK, _
                MessageBoxIcon.Exclamation)
        End Try

        ' Display the result.
        dgvBooks.DataSource = data_table

        ' Set column alignments.
        For c As Integer = 0 To dgvBooks.Columns.Count - 1
            Select Case dgvBooks.Columns(c).ValueType.ToString
                Case "System.Decimal"  ' Currency.
                    dgvBooks.Columns(c).DefaultCellStyle.Format = "c"
                    dgvBooks.Columns(c).DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
                Case "System.DateTime" ' Date.
                    dgvBooks.Columns(c).DefaultCellStyle.Format = "d"
                    dgvBooks.Columns(c).DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
                Case "System.String"
                    dgvBooks.Columns(c).DefaultCellStyle.Format = ""
                    dgvBooks.Columns(c).DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft
                Case "System.Single"    ' One digit after the decimal point.
                    dgvBooks.Columns(c).DefaultCellStyle.Format = ".0"
                    dgvBooks.Columns(c).DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
                Case Else       ' Probably number.
                    dgvBooks.Columns(c).DefaultCellStyle.Format = ""
                    dgvBooks.Columns(c).DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight
            End Select
        Next c

        ' Close the connection.
        m_Connection.Close()
    End Sub

    ' Save the application's configuration.
    Private Sub LoadSettings()
        ' Load the column visibilities.
        For i As Integer = 0 To dgvBooks.Columns.Count - 1
            Dim value As String = GetSetting( _
                Application.ProductName, _
                "ColumnVisibilities", _
                "Column" & i.ToString(), _
                "True" _
            )
            dgvBooks.Columns(i).Visible = Boolean.Parse(value)
        Next i

        ' Load the size.
        Dim wid As Integer = _
            Integer.Parse(GetSetting( _
                Application.ProductName, _
                "General", _
                "Width", _
                Me.Width.ToString()))
        Dim hgt As Integer = _
            Integer.Parse(GetSetting( _
                Application.ProductName, _
                "General", _
                "Height", _
                Me.Height.ToString()))
        Me.Size = New Size(wid, hgt)

        ' Load the query combo box's items.
        cboQuery.Items.Clear()
        For i As Integer = 0 To 10000
            Dim query As String = GetSetting( _
                Application.ProductName, _
                "Queries", _
                "Query" & i.ToString(), _
                "")
            If query.Length = 0 Then Exit For
            cboQuery.Items.Add(query)
        Next i
    End Sub

    ' Save the application's configuration.
    Private Sub SaveSettings()
        ' Do nothing if we're not loaded yet.
        If Not m_Loaded Then Exit Sub

        ' Save the column visibilities.
        For i As Integer = 0 To dgvBooks.Columns.Count - 1
            SaveSetting( _
                Application.ProductName, _
                "ColumnVisibilities", _
                "Column" & i.ToString(), _
                dgvBooks.Columns(i).Visible.ToString())
        Next i

        ' Save the size.
        SaveSetting( _
            Application.ProductName, _
            "General", _
            "Width", _
            Me.Width.ToString())
        SaveSetting( _
            Application.ProductName, _
            "General", _
            "Height", _
            Me.Height.ToString())

        ' Delete existing queries.
        If GetSetting(Application.ProductName, "Queries", "Query0", "").Length > 0 Then
            DeleteSetting(Application.ProductName, "Queries")
        End If

        ' Save the query combo box's items.
        For i As Integer = 0 To cboQuery.Items.Count - 1
            SaveSetting( _
                Application.ProductName, _
                "Queries", _
                "Query" & i.ToString(), _
                cboQuery.Items.Item(i).ToString())
        Next i
    End Sub

    ' Close.
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        m_Connection.Dispose()
        Me.Close()
    End Sub

    ' Save the settings to save the size.
    Private Sub Form1_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        SaveSettings()
    End Sub

    ' Let the user select the fields to display.
    Private Sub mnuDataSelectFields_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDataSelectFields.Click
        Dim dlg As New dlgSelectFields
        dlg.TheDataGridView = dgvBooks
        If dlg.ShowDialog() = Windows.Forms.DialogResult.OK Then
            ' Save the settings.
            SaveSettings()
        End If
    End Sub

    ' Open the selected book's Web page.
    Private Const URL_FIELD As Integer = 1
    Private Sub btnOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOpen.Click
        Dim selected_url As String = dgvBooks.SelectedRows.Item(0).Cells(URL_FIELD).Value.ToString
        System.Diagnostics.Process.Start(selected_url)
    End Sub

    ' Open the selected book's Web page.
    Private Sub dgvBooks_CellDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dgvBooks.CellDoubleClick
        Dim selected_url As String = dgvBooks.Rows(e.RowIndex).Cells(URL_FIELD).Value.ToString
        System.Diagnostics.Process.Start(selected_url)
    End Sub

    ' Execute the user's query.
    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        If cboQuery.Text.Length > 0 Then SelectData(cboQuery.Text)
    End Sub

    ' Save the current query.
    Private Sub SaveQuery(ByVal query As String)
        ' See if the query is already in the combo box's list.
        For i As Integer = 0 To cboQuery.Items.Count - 1
            If cboQuery.Items.Item(i).ToString() = query Then
                ' The query is already here. Do nothing.
                Exit Sub
            End If
        Next i

        ' Add the query to the list.
        cboQuery.Items.Add(query)

        ' Save the settings.
        SaveSettings()
    End Sub
End Class
